@@ -431,7 +431,7 @@ int setup_files(struct thread_data *td)
 	 */
 	if (need_extend) {
 		temp_stall_ts = 1;
-		log_info("%s: Laying out IO file(s) (%u files / %LuMiB)\n",
+		log_info("%s: Laying out IO file(s) (%u file(s) / %LuMiB)\n",
 			td->o.name, need_extend, extend_size >> 20);
 
 		for_each_file(td, f, i) {
@@ -454,7 +454,12 @@ int setup_files(struct thread_data *td)
 	if (!td->o.zone_size)
 		td->o.zone_size = td->o.size;
 
-	td->total_io_size = td->o.size * td->o.loops;
+	/*
+	 * iolog already set the total io size, if we read back
+	 * stored entries.
+	 */
+	if (!td->o.read_iolog_file)
+		td->total_io_size = td->o.size * td->o.loops;
 	return 0;
 err_offset:
 	log_err("%s: you need to specify valid offset=\n", td->o.name);
