@@ -55,7 +55,7 @@ static void mark_random_map(struct thread_data *td, struct io_u *io_u)
 		 * If we have a mixed random workload, we may
 		 * encounter blocks we already did IO to.
 		 */
-		if (!td->o.ddir_nr && !random_map_free(td, f, block))
+		if (!td->o.ddir_nr == 1 && !random_map_free(td, f, block))
 			break;
 
 		idx = RAND_MAP_IDX(td, f, block);
@@ -162,10 +162,11 @@ static int get_next_offset(struct thread_data *td, struct io_u *io_u)
 		if (get_next_rand_offset(td, f, ddir, &b))
 			return 1;
 	} else {
-		if (f->last_pos >= f->real_file_size)
-			return 1;
-
-		b = f->last_pos / td->o.min_bs[ddir];
+		if (f->last_pos >= f->real_file_size) {
+			if (!td_random(td) || get_next_rand_offset(td, f, ddir, &b))
+				return 1;
+		} else
+			b = (f->last_pos - f->file_offset) / td->o.min_bs[ddir];
 	}
 
 	io_u->offset = (b * td->o.min_bs[ddir]) + f->file_offset;
@@ -190,6 +191,9 @@ static unsigned int get_next_buflen(struct thread_data *td, struct io_u *io_u)
 			buflen = (buflen + td->o.min_bs[ddir] - 1) & ~(td->o.min_bs[ddir] - 1);
 	}
 
+	if (io_u->offset + buflen > io_u->file->real_file_size)
+		buflen = td->o.min_bs[ddir];
+
 	return buflen;
 }
 
@@ -309,12 +313,6 @@ void requeue_io_u(struct thread_data *td, struct io_u **io_u)
 
 static int fill_io_u(struct thread_data *td, struct io_u *io_u)
 {
-	/*
-	 * If using an iolog, grab next piece if any available.
-	 */
-	if (td->o.read_iolog_file)
-		return read_iolog_get(td, io_u);
-
 	/*
 	 * see if it's time to sync
 	 */
@@ -347,6 +345,9 @@ static int fill_io_u(struct thread_data *td, struct io_u *io_u)
 	if (!io_u->buflen)
 		return 1;
 
+	if (io_u->offset + io_u->buflen > io_u->file->real_file_size)
+		return 1;
+
 	/*
 	 * mark entry before potentially trimming io_u
 	 */
@@ -357,9 +358,7 @@ static int fill_io_u(struct thread_data *td, struct io_u *io_u)
 	 * If using a write iolog, store this entry.
 	 */
 out:
-	if (td->o.write_iolog_file)
-		write_iolog_put(td, io_u);
-
+	log_io_u(td, io_u);
 	return 0;
 }
 
@@ -580,6 +579,50 @@ static struct fio_file *find_next_new_file(struct thread_data *td)
 	return f;
 }
 
+static int set_io_u_file(struct thread_data *td, struct io_u *io_u)
+{
+	struct fio_file *f;
+
+	do {
+		f = get_next_file(td);
+		if (!f)
+			return 1;
+
+set_file:
+		io_u->file = f;
+		get_file(f);
+
+		if (!fill_io_u(td, io_u))
+			break;
+
+		/*
+		 * td_io_close() does a put_file() as well, so no need to
+		 * do that here.
+		 */
+		io_u->file = NULL;
+		td_io_close_file(td, f);
+		f->flags |= FIO_FILE_DONE;
+		td->nr_done_files++;
+
+		/*
+		 * probably not the right place to do this, but see
+		 * if we need to open a new file
+		 */
+		if (td->nr_open_files < td->o.open_files &&
+		    td->o.open_files != td->o.nr_files) {
+			f = find_next_new_file(td);
+
+			if (!f || td_io_open_file(td, f))
+				return 1;
+
+			goto set_file;
+		}
+	} while (1);
+
+	return 0;
+}
+
+
 struct io_u *__get_io_u(struct thread_data *td)
 {
 	struct io_u *io_u = NULL;
@@ -616,7 +659,6 @@ struct io_u *get_io_u(struct thread_data *td)
 {
 	struct fio_file *f;
 	struct io_u *io_u;
-	int ret;
 
 	io_u = __get_io_u(td);
 	if (!io_u)
@@ -628,52 +670,21 @@ struct io_u *get_io_u(struct thread_data *td)
 	if (io_u->file)
 		goto out;
 
-	do {
-		f = get_next_file(td);
-		if (!f) {
-			put_io_u(td, io_u);
-			return NULL;
-		}
-
-set_file:
-		io_u->file = f;
-		get_file(f);
-
-		if (!fill_io_u(td, io_u))
-			break;
-
-		/*
-		 * td_io_close() does a put_file() as well, so no need to
-		 * do that here.
-		 */
-		io_u->file = NULL;
-		td_io_close_file(td, f);
-		f->flags |= FIO_FILE_DONE;
-		td->nr_done_files++;
-
-		/*
-		 * probably not the right place to do this, but see
-		 * if we need to open a new file
-		 */
-		if (td->nr_open_files < td->o.open_files &&
-		    td->o.open_files != td->o.nr_files) {
-			f = find_next_new_file(td);
-
-			if (!f || (ret = td_io_open_file(td, f))) {
-				put_io_u(td, io_u);
-				return NULL;
-			}
-			goto set_file;
-		}
-	} while (1);
-
-	assert(io_u->file->flags & FIO_FILE_OPEN);
+	/*
+	 * If using an iolog, grab next piece if any available.
+	 */
+	if (td->o.read_iolog_file) {
+		if (read_iolog_get(td, io_u))
+			goto err_put;
+	} else if (set_io_u_file(td, io_u))
+		goto err_put;
+	
+	f = io_u->file;
+	assert(f->flags & FIO_FILE_OPEN);
 
 	if (io_u->ddir != DDIR_SYNC) {
-		if (!io_u->buflen) {
-			put_io_u(td, io_u);
-			return NULL;
-		}
+		if (!io_u->buflen)
+			goto err_put;
 
 		f->last_pos = io_u->offset + io_u->buflen;
 
@@ -689,13 +700,13 @@ out:
 	io_u->xfer_buf = io_u->buf;
 	io_u->xfer_buflen = io_u->buflen;
 
-	if (td_io_prep(td, io_u)) {
-		put_io_u(td, io_u);
-		return NULL;
+	if (!td_io_prep(td, io_u)) {
+		fio_gettime(&io_u->start_time, NULL);
+		return io_u;
 	}
-
-	fio_gettime(&io_u->start_time, NULL);
-	return io_u;
+err_put:
+	put_io_u(td, io_u);
+	return NULL;
 }
 
 void io_u_log_error(struct thread_data *td, struct io_u *io_u)
