@@ -192,6 +192,7 @@ int td_io_queue(struct thread_data *td, struct io_u *io_u)
 
 	if (td->io_ops->flags & FIO_SYNCIO) {
 		fio_gettime(&io_u->issue_time, NULL);
+		memcpy(&td->last_issue, &io_u->issue_time, sizeof(struct timeval));
 
 		/*
 		 * for a sync engine, set the timeout upfront
@@ -220,6 +221,7 @@ int td_io_queue(struct thread_data *td, struct io_u *io_u)
 
 	if ((td->io_ops->flags & FIO_SYNCIO) == 0) {
 		fio_gettime(&io_u->issue_time, NULL);
+		memcpy(&td->last_issue, &io_u->issue_time, sizeof(struct timeval));
 
 		/*
 		 * async engine, set the timeout here
@@ -277,7 +279,7 @@ int td_io_open_file(struct thread_data *td, struct fio_file *f)
 
 	f->last_free_lookup = 0;
 	f->last_completed_pos = 0;
-	f->last_pos = 0;
+	f->last_pos = f->file_offset;
 	f->flags |= FIO_FILE_OPEN;
 	f->flags &= ~FIO_FILE_CLOSING;
 
@@ -307,6 +309,7 @@ int td_io_open_file(struct thread_data *td, struct fio_file *f)
 		memset(f->file_map, 0, f->num_maps * sizeof(long));
 
 done:
+	log_file(td, f, FIO_LOG_OPEN_FILE);
 	td->nr_open_files++;
 	get_file(f);
 	return 0;
@@ -318,6 +321,9 @@ err:
 
 void td_io_close_file(struct thread_data *td, struct fio_file *f)
 {
+	if (!(f->flags & FIO_FILE_CLOSING))
+		log_file(td, f, FIO_LOG_CLOSE_FILE);
+
 	/*
 	 * mark as closing, do real close when last io on it has completed
 	 */
