@@ -27,9 +27,11 @@
 #ifndef FIO_HAVE_FADVISE
 #define fadvise(fd, off, len, advice)	(0)
 
+#ifndef POSIX_FADV_DONTNEED
 #define POSIX_FADV_DONTNEED	(0)
 #define POSIX_FADV_SEQUENTIAL	(0)
 #define POSIX_FADV_RANDOM	(0)
+#endif
 #endif /* FIO_HAVE_FADVISE */
 
 #ifndef FIO_HAVE_CPU_AFFINITY
@@ -49,7 +51,9 @@
 
 #ifndef FIO_HAVE_HUGETLB
 #define SHM_HUGETLB			0
+#ifndef FIO_HUGE_PAGE
 #define FIO_HUGE_PAGE			0
+#endif
 #else
 #ifndef FIO_HUGE_PAGE
 #define FIO_HUGE_PAGE			4194304
@@ -60,4 +64,15 @@
 #define fio_lookup_raw(dev, majdev, mindev)	1
 #endif
 
+#ifndef FIO_HAVE_BLKTRACE
+static inline int is_blktrace(const char *fname)
+{
+	return 0;
+}
+static inline int load_blktrace(struct thread_data *td, const char *fname)
+{
+	return 1;
+}
+#endif
+
 #endif
